@@ -38,13 +38,13 @@ _PKG_TARGETS = {
     'Debian': ['python-plist', 'apg'],
     'RedHat': ['units', 'zsh-html'],
     'FreeBSD': ['aalib', 'pth'],
-    'Suse': ['aalib', 'rpm-python'],
+    'Suse': ['aalib', 'htop'],
     'MacOS': ['libpng', 'jpeg'],
-    'Windows': ['firefox', '7zip'],
+    'Windows': ['putty', '7zip'],
 }
 
 _PKG_CAP_TARGETS = {
-    'Suse': [('w3m_ssl', 'w3m')],
+    'Suse': [('perl(ZNC)', 'znc-perl')],
 }
 
 _PKG_TARGETS_32 = {
@@ -167,7 +167,6 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
             self.run_function('pkg.refresh_db')
             __testcontext__['refresh'] = True
 
-    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
     @requires_system_grains
     def test_pkg_001_installed(self, grains=None):
         '''
@@ -198,7 +197,6 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
         ret = self.run_state('pkg.removed', name=target)
         self.assertSaltTrueReturn(ret)
 
-    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
     @requires_system_grains
     def test_pkg_002_installed_with_version(self, grains=None):
         '''
@@ -246,7 +244,6 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
         ret = self.run_state('pkg.removed', name=target)
         self.assertSaltTrueReturn(ret)
 
-    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
     @requires_system_grains
     def test_pkg_003_installed_multipkg(self, grains=None):
         '''
@@ -268,17 +265,21 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
         # If this assert fails, we need to find new targets, this test needs to
         # be able to test successful installation of packages, so these
         # packages need to not be installed before we run the states below
-        self.assertFalse(any(version.values()))
+        try:
+            self.assertFalse(any(version.values()))
+        except AssertionError:
+            self.assertSaltTrueReturn(self.run_state('pkg.removed', name=None, pkgs=pkg_targets))
 
-        ret = self.run_state('pkg.installed',
-                             name=None,
-                             pkgs=pkg_targets,
-                             refresh=False)
-        self.assertSaltTrueReturn(ret)
-        ret = self.run_state('pkg.removed', name=None, pkgs=pkg_targets)
-        self.assertSaltTrueReturn(ret)
+        try:
+            ret = self.run_state('pkg.installed',
+                                 name=None,
+                                 pkgs=pkg_targets,
+                                 refresh=False)
+            self.assertSaltTrueReturn(ret)
+        finally:
+            ret = self.run_state('pkg.removed', name=None, pkgs=pkg_targets)
+            self.assertSaltTrueReturn(ret)
 
-    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
     @requires_system_grains
     def test_pkg_004_installed_multipkg_with_version(self, grains=None):
         '''
@@ -319,15 +320,16 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
 
         pkgs = [{pkg_targets[0]: version}, pkg_targets[1]]
 
-        ret = self.run_state('pkg.installed',
-                             name=None,
-                             pkgs=pkgs,
-                             refresh=False)
-        self.assertSaltTrueReturn(ret)
-        ret = self.run_state('pkg.removed', name=None, pkgs=pkg_targets)
-        self.assertSaltTrueReturn(ret)
+        try:
+            ret = self.run_state('pkg.installed',
+                                 name=None,
+                                 pkgs=pkgs,
+                                 refresh=False)
+            self.assertSaltTrueReturn(ret)
+        finally:
+            ret = self.run_state('pkg.removed', name=None, pkgs=pkg_targets)
+            self.assertSaltTrueReturn(ret)
 
-    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
     @requires_system_grains
     def test_pkg_005_installed_32bit(self, grains=None):
         '''
@@ -364,7 +366,6 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
             ret = self.run_state('pkg.removed', name=target)
             self.assertSaltTrueReturn(ret)
 
-    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
     @requires_system_grains
     def test_pkg_006_installed_32bit_with_version(self, grains=None):
         '''
@@ -412,7 +413,6 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
         ret = self.run_state('pkg.removed', name=target)
         self.assertSaltTrueReturn(ret)
 
-    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
     @requires_system_grains
     def test_pkg_007_with_dot_in_pkgname(self, grains=None):
         '''
@@ -443,7 +443,6 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
         ret = self.run_state('pkg.removed', name=target)
         self.assertSaltTrueReturn(ret)
 
-    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
     @requires_system_grains
     def test_pkg_008_epoch_in_version(self, grains=None):
         '''
@@ -478,7 +477,8 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
         self.assertSaltTrueReturn(ret)
 
     @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
-    def test_pkg_009_latest_with_epoch(self):
+    @requires_system_grains
+    def test_pkg_009_latest_with_epoch(self, grains=None):
         '''
         This tests for the following issue:
         https://github.com/saltstack/salt/issues/31014
@@ -489,14 +489,18 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
         if not pkgmgr_avail(self.run_function, self.run_function('grains.items')):
             self.skipTest('Package manager is not available')
 
+        test_name = 'bash-completion'
+        if grains.get('os') == 'Amazon' and grains.get('osmajorrelease') != 2:
+            test_name = 'bash-doc'
+
         ret = self.run_state('pkg.installed',
-                             name='bash-completion',
+                             name=test_name,
                              refresh=False)
         self.assertSaltTrueReturn(ret)
 
-    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
     @requires_salt_modules('pkg.info_installed')
-    def test_pkg_010_latest_with_epoch_and_info_installed(self):
+    @requires_system_grains
+    def test_pkg_010_latest_with_epoch_and_info_installed(self, grains=None):
         '''
         Need to check to ensure the package has been
         installed after the pkg_latest_epoch sls
@@ -509,12 +513,14 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
             self.skipTest('Package manager is not available')
 
         package = 'bash-completion'
+        if grains.get('os') == 'Amazon' and grains.get('osmajorrelease') != 2:
+            package = 'bash-doc'
+
         pkgquery = 'version'
 
         ret = self.run_function('pkg.info_installed', [package])
         self.assertTrue(pkgquery in six.text_type(ret))
 
-    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
     @requires_system_grains
     def test_pkg_011_latest(self, grains=None):
         '''
@@ -546,7 +552,6 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
         ret = self.run_state('pkg.removed', name=target)
         self.assertSaltTrueReturn(ret)
 
-    @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
     @requires_system_grains
     def test_pkg_012_latest_only_upgrade(self, grains=None):
         '''
@@ -732,6 +737,7 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
             ret = self.run_state('pkg.removed', name=target)
             self.assertSaltTrueReturn(ret)
 
+    @requires_system_grains
     def test_pkg_014_installed_missing_release(self, grains=None):  # pylint: disable=unused-argument
         '''
         Tests that a version number missing the release portion still resolves
@@ -824,12 +830,14 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
         self.assertFalse(version)
         self.assertFalse(realver)
 
-        ret = self.run_state('pkg.installed', name=target, refresh=False, resolve_capabilities=True, test=True)
-        self.assertInSaltComment("The following packages would be installed/updated: {0}".format(realpkg), ret)
-        ret = self.run_state('pkg.installed', name=target, refresh=False, resolve_capabilities=True)
-        self.assertSaltTrueReturn(ret)
-        ret = self.run_state('pkg.removed', name=realpkg)
-        self.assertSaltTrueReturn(ret)
+        try:
+            ret = self.run_state('pkg.installed', name=target, refresh=False, resolve_capabilities=True, test=True)
+            self.assertInSaltComment("The following packages would be installed/updated: {0}".format(realpkg), ret)
+            ret = self.run_state('pkg.installed', name=target, refresh=False, resolve_capabilities=True)
+            self.assertSaltTrueReturn(ret)
+        finally:
+            ret = self.run_state('pkg.removed', name=realpkg)
+            self.assertSaltTrueReturn(ret)
 
     @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
     @requires_system_grains
@@ -856,18 +864,22 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
         self.assertFalse(version)
         self.assertFalse(realver)
 
-        # install the package already
-        ret = self.run_state('pkg.installed', name=realpkg, refresh=False)
+        try:
+            # install the package
+            ret = self.run_state('pkg.installed', name=realpkg, refresh=False)
+            self.assertSaltTrueReturn(ret)
 
-        ret = self.run_state('pkg.installed', name=target, refresh=False, resolve_capabilities=True, test=True)
-        self.assertInSaltComment("All specified packages are already installed", ret)
+            # Try to install again. Nothing should be installed this time.
+            ret = self.run_state('pkg.installed', name=target, refresh=False, resolve_capabilities=True, test=True)
+            self.assertInSaltComment("All specified packages are already installed", ret)
 
-        ret = self.run_state('pkg.installed', name=target, refresh=False, resolve_capabilities=True)
-        self.assertSaltTrueReturn(ret)
+            ret = self.run_state('pkg.installed', name=target, refresh=False, resolve_capabilities=True)
+            self.assertSaltTrueReturn(ret)
 
-        self.assertInSaltComment("packages are already installed", ret)
-        ret = self.run_state('pkg.removed', name=realpkg)
-        self.assertSaltTrueReturn(ret)
+            self.assertInSaltComment("packages are already installed", ret)
+        finally:
+            ret = self.run_state('pkg.removed', name=realpkg)
+            self.assertSaltTrueReturn(ret)
 
     @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
     @requires_system_grains
@@ -893,8 +905,8 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
         # Make sure that we have targets that match the os_family. If this
         # fails then the _PKG_TARGETS dict above needs to have an entry added,
         # with two packages that are not installed before these tests are run
-        self.assertTrue(bool(pkg_cap_targets))
-        self.assertTrue(bool(pkg_targets))
+        self.assertTrue(pkg_cap_targets)
+        self.assertTrue(pkg_targets)
 
         if os_family == 'Arch':
             for idx in range(13):
@@ -912,34 +924,36 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
         # If this assert fails, we need to find new targets, this test needs to
         # be able to test successful installation of packages, so these
         # packages need to not be installed before we run the states below
-        self.assertTrue(bool(version))
-        self.assertTrue(bool(realver))
+        self.assertTrue(version)
+        self.assertTrue(realver)
 
-        pkgs = [{pkg_targets[0]: version}, pkg_targets[1], {capability: realver}]
-        ret = self.run_state('pkg.installed',
-                             name='test_pkg_cap_003_installed_multipkg_with_version-install',
-                             pkgs=pkgs,
-                             refresh=False)
-        self.assertSaltFalseReturn(ret)
+        try:
+            pkgs = [{pkg_targets[0]: version}, pkg_targets[1], {capability: realver}]
+            ret = self.run_state('pkg.installed',
+                                 name='test_pkg_cap_003_installed_multipkg_with_version-install',
+                                 pkgs=pkgs,
+                                 refresh=False)
+            self.assertSaltFalseReturn(ret)
 
-        ret = self.run_state('pkg.installed',
-                             name='test_pkg_cap_003_installed_multipkg_with_version-install-capability',
-                             pkgs=pkgs,
-                             refresh=False, resolve_capabilities=True, test=True)
-        self.assertInSaltComment("packages would be installed/updated", ret)
-        self.assertInSaltComment("{0}={1}".format(realpkg, realver), ret)
+            ret = self.run_state('pkg.installed',
+                                 name='test_pkg_cap_003_installed_multipkg_with_version-install-capability',
+                                 pkgs=pkgs,
+                                 refresh=False, resolve_capabilities=True, test=True)
+            self.assertInSaltComment("packages would be installed/updated", ret)
+            self.assertInSaltComment("{0}={1}".format(realpkg, realver), ret)
 
-        ret = self.run_state('pkg.installed',
-                             name='test_pkg_cap_003_installed_multipkg_with_version-install-capability',
-                             pkgs=pkgs,
-                             refresh=False, resolve_capabilities=True)
-        self.assertSaltTrueReturn(ret)
-        cleanup_pkgs = pkg_targets
-        cleanup_pkgs.append(realpkg)
-        ret = self.run_state('pkg.removed',
-                             name='test_pkg_cap_003_installed_multipkg_with_version-remove',
-                             pkgs=cleanup_pkgs)
-        self.assertSaltTrueReturn(ret)
+            ret = self.run_state('pkg.installed',
+                                 name='test_pkg_cap_003_installed_multipkg_with_version-install-capability',
+                                 pkgs=pkgs,
+                                 refresh=False, resolve_capabilities=True)
+            self.assertSaltTrueReturn(ret)
+            cleanup_pkgs = pkg_targets
+            cleanup_pkgs.append(realpkg)
+        finally:
+            ret = self.run_state('pkg.removed',
+                                 name='test_pkg_cap_003_installed_multipkg_with_version-remove',
+                                 pkgs=cleanup_pkgs)
+            self.assertSaltTrueReturn(ret)
 
     @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
     @requires_system_grains
@@ -967,17 +981,18 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
         self.assertFalse(version)
         self.assertFalse(realver)
 
-        ret = self.run_state('pkg.latest', name=target, refresh=False, resolve_capabilities=True, test=True)
-        self.assertInSaltComment("The following packages would be installed/upgraded: {0}".format(realpkg), ret)
-        ret = self.run_state('pkg.latest', name=target, refresh=False, resolve_capabilities=True)
-        self.assertSaltTrueReturn(ret)
+        try:
+            ret = self.run_state('pkg.latest', name=target, refresh=False, resolve_capabilities=True, test=True)
+            self.assertInSaltComment("The following packages would be installed/upgraded: {0}".format(realpkg), ret)
+            ret = self.run_state('pkg.latest', name=target, refresh=False, resolve_capabilities=True)
+            self.assertSaltTrueReturn(ret)
 
-        ret = self.run_state('pkg.latest', name=target, refresh=False, resolve_capabilities=True)
-        self.assertSaltTrueReturn(ret)
-        self.assertInSaltComment("is already up-to-date", ret)
-
-        ret = self.run_state('pkg.removed', name=realpkg)
-        self.assertSaltTrueReturn(ret)
+            ret = self.run_state('pkg.latest', name=target, refresh=False, resolve_capabilities=True)
+            self.assertSaltTrueReturn(ret)
+            self.assertInSaltComment("is already up-to-date", ret)
+        finally:
+            ret = self.run_state('pkg.removed', name=realpkg)
+            self.assertSaltTrueReturn(ret)
 
     @skipIf(salt.utils.platform.is_windows(), 'minion is windows')
     @requires_system_grains
@@ -1038,20 +1053,87 @@ class PkgTest(ModuleCase, SaltReturnAssertsMixin):
         self.assertFalse(version)
         self.assertFalse(realver)
 
-        ret = self.run_state('pkg.installed', name=target,
-                             refresh=False, resolve_capabilities=True)
+        try:
+            ret = self.run_state('pkg.installed', name=target,
+                                 refresh=False, resolve_capabilities=True)
+            self.assertSaltTrueReturn(ret)
+            ret = self.run_state('pkg.uptodate',
+                                 name='test_pkg_cap_006_uptodate',
+                                 pkgs=[target],
+                                 refresh=False,
+                                 resolve_capabilities=True)
+            self.assertSaltTrueReturn(ret)
+            self.assertInSaltComment("System is already up-to-date", ret)
+        finally:
+            ret = self.run_state('pkg.removed', name=realpkg)
+            self.assertSaltTrueReturn(ret)
+
+    @skipIf(True, 'WAR ROOM TEMPORARY SKIP')            # needs to be rewritten to allow for dnf on Fedora 30 and RHEL 8
+    @requires_salt_modules('pkg.hold', 'pkg.unhold')
+    @requires_system_grains
+    def test_pkg_015_installed_held(self, grains=None):  # pylint: disable=unused-argument
+        '''
+        Tests that a package can be held even when the package is already installed.
+        '''
+        os_family = grains.get('os_family', '')
+
+        if os_family.lower() != 'redhat' and os_family.lower() != 'debian':
+            self.skipTest('Test only runs on RedHat or Debian family')
+
+        pkg_targets = _PKG_TARGETS.get(os_family, [])
+
+        if os_family.lower() == 'redhat':
+            # If we're in the Red Hat family first we ensure that
+            # the yum-plugin-versionlock package is installed
+            ret = self.run_state(
+                'pkg.installed',
+                name='yum-plugin-versionlock',
+                refresh=False,
+            )
+            self.assertSaltTrueReturn(ret)
+
+        # Make sure that we have targets that match the os_family. If this
+        # fails then the _PKG_TARGETS dict above needs to have an entry added,
+        # with two packages that are not installed before these tests are run
+        self.assertTrue(pkg_targets)
+
+        target = pkg_targets[0]
+
+        # First we ensure that the package is installed
+        ret = self.run_state(
+            'pkg.installed',
+            name=target,
+            refresh=False,
+        )
         self.assertSaltTrueReturn(ret)
-        ret = self.run_state('pkg.uptodate',
-                             name='test_pkg_cap_006_uptodate',
-                             pkgs=[target],
-                             refresh=False,
-                             resolve_capabilities=True)
-        self.assertSaltTrueReturn(ret)
-        self.assertInSaltComment("System is already up-to-date", ret)
-        ret = self.run_state('pkg.removed', name=realpkg)
-        self.assertSaltTrueReturn(ret)
-        ret = self.run_state('pkg.uptodate',
-                             name='test_pkg_cap_006_uptodate',
-                             refresh=False,
-                             test=True)
-        self.assertInSaltComment("System update will be performed", ret)
+
+        # Then we check that the package is now held
+        ret = self.run_state(
+            'pkg.installed',
+            name=target,
+            hold=True,
+            refresh=False,
+        )
+
+        # changes from pkg.hold for Red Hat family are different
+        if os_family.lower() == 'redhat':
+            target_changes = {'new': 'hold', 'old': ''}
+        elif os_family.lower() == 'debian':
+            target_changes = {'new': 'hold', 'old': 'install'}
+
+        try:
+            tag = 'pkg_|-{0}_|-{0}_|-installed'.format(target)
+            self.assertSaltTrueReturn(ret)
+            self.assertIn(tag, ret)
+            self.assertIn('changes', ret[tag])
+            self.assertIn(target, ret[tag]['changes'])
+            self.assertEqual(ret[tag]['changes'][target], target_changes)
+        finally:
+            # Clean up, unhold package and remove
+            self.run_function('pkg.unhold', name=target)
+            ret = self.run_state('pkg.removed', name=target)
+            self.assertSaltTrueReturn(ret)
+            if os_family.lower() == 'redhat':
+                ret = self.run_state('pkg.removed',
+                                     name='yum-plugin-versionlock')
+                self.assertSaltTrueReturn(ret)
