@@ -31,6 +31,7 @@ import salt.utils.files
 import salt.utils.platform
 import salt.utils.win_functions
 import salt.utils.yaml
+import salt.ext.six
 
 import salt.utils.gitfs
 from salt.utils.gitfs import (
@@ -62,8 +63,10 @@ log = logging.getLogger(__name__)
 
 TMP_SOCK_DIR = tempfile.mkdtemp(dir=TMP)
 TMP_REPO_DIR = os.path.join(TMP, 'gitfs_root')
+TMP_FILE_URI = 'file://'
 if salt.utils.platform.is_windows():
     TMP_REPO_DIR = TMP_REPO_DIR.replace('\\', '/')
+    TMP_FILE_URI = TMP_FILE_URI + '/'
 INTEGRATION_BASE_FILES = os.path.join(FILES, 'file', 'base')
 UNICODE_FILENAME = 'питон.txt'
 UNICODE_DIRNAME = UNICODE_ENVNAME = 'соль'
@@ -71,7 +74,7 @@ TAG_NAME = 'mytag'
 
 OPTS = {
     'sock_dir': TMP_SOCK_DIR,
-    'gitfs_remotes': ['file://' + TMP_REPO_DIR],
+    'gitfs_remotes': [TMP_FILE_URI + TMP_REPO_DIR],
     'gitfs_root': '',
     'fileserver_backend': ['gitfs'],
     'gitfs_base': 'master',
@@ -144,6 +147,9 @@ class GitfsConfigTestCase(TestCase, LoaderModuleMockMixin):
             try:
                 shutil.rmtree(path, onerror=_rmtree_error)
             except OSError as exc:
+                if exc.errno == errno.EACCES:
+                    log.error("Access error removeing file %s", path)
+                    continue
                 if exc.errno != errno.EEXIST:
                     raise
 
@@ -160,18 +166,18 @@ class GitfsConfigTestCase(TestCase, LoaderModuleMockMixin):
 
             gitfs_remotes:
 
-              - file://tmp/repo1:
+              - {0}tmp/repo1:
                 - saltenv:
                   - foo:
                     - ref: foo_branch
                     - root: foo_root
 
-              - file://tmp/repo2:
+              - {0}tmp/repo2:
                 - mountpoint: repo2
                 - saltenv:
                   - baz:
                     - mountpoint: abc
-        ''')
+        '''.format(TMP_FILE_URI))
         with patch.dict(gitfs.__opts__, salt.utils.yaml.safe_load(opts_override)):
             git_fs = salt.utils.gitfs.GitFS(
                 gitfs.__opts__,
@@ -287,7 +293,7 @@ class GitFSTestFuncs(object):
         Test the per_remote ref_types config option, using a different
         ref_types setting than the global test.
         '''
-        remotes = [{'file://' + TMP_REPO_DIR: [{'ref_types': ['tag']}]}]
+        remotes = [{TMP_FILE_URI + TMP_REPO_DIR: [{'ref_types': ['tag']}]}]
         with patch.dict(gitfs.__opts__, {'gitfs_remotes': remotes}):
             gitfs.update()
             ret = gitfs.envs(ignore_cache=True)
@@ -325,11 +331,11 @@ class GitFSTestFuncs(object):
         opts = salt.utils.yaml.safe_load(textwrap.dedent('''\
             gitfs_disable_saltenv_mapping: True
             gitfs_remotes:
-              - file://{0}:
+              - {0}:
                 - saltenv:
                   - bar:
                     - ref: somebranch
-            '''.format(TMP_REPO_DIR)))
+            '''.format(TMP_FILE_URI + TMP_REPO_DIR)))
         with patch.dict(gitfs.__opts__, opts):
             gitfs.update()
             ret = gitfs.envs(ignore_cache=True)
@@ -345,13 +351,13 @@ class GitFSTestFuncs(object):
         '''
         opts = salt.utils.yaml.safe_load(textwrap.dedent('''\
             gitfs_remotes:
-              - file://{0}:
+              - {0}:
                 - disable_saltenv_mapping: True
 
             gitfs_saltenv:
               - hello:
                 - ref: somebranch
-            '''.format(TMP_REPO_DIR)))
+            '''.format(TMP_FILE_URI + TMP_REPO_DIR)))
         with patch.dict(gitfs.__opts__, opts):
             gitfs.update()
             ret = gitfs.envs(ignore_cache=True)
@@ -367,12 +373,12 @@ class GitFSTestFuncs(object):
         '''
         opts = salt.utils.yaml.safe_load(textwrap.dedent('''\
             gitfs_remotes:
-              - file://{0}:
+              - {0}:
                 - disable_saltenv_mapping: True
                 - saltenv:
                   - world:
                     - ref: somebranch
-            '''.format(TMP_REPO_DIR)))
+            '''.format(TMP_FILE_URI + TMP_REPO_DIR)))
         with patch.dict(gitfs.__opts__, opts):
             gitfs.update()
             ret = gitfs.envs(ignore_cache=True)
@@ -391,7 +397,9 @@ class GitFSTestBase(object):
         try:
             shutil.rmtree(TMP_REPO_DIR)
         except OSError as exc:
-            if exc.errno != errno.ENOENT:
+            if exc.errno == errno.EACCES:
+                log.error("Access error removeing file %s", TMP_REPO_DIR)
+            elif exc.errno != errno.ENOENT:
                 raise
         shutil.copytree(INTEGRATION_BASE_FILES, TMP_REPO_DIR + '/')
 
@@ -399,8 +407,10 @@ class GitFSTestBase(object):
 
         username_key = str('USERNAME')
         orig_username = os.environ.get(username_key)
+        environ_copy = os.environ.copy()
         try:
             if username_key not in os.environ:
+
                 try:
                     if salt.utils.platform.is_windows():
                         os.environ[username_key] = \
@@ -428,10 +438,8 @@ class GitFSTestBase(object):
             if hasattr(repo, 'close'):
                 repo.close()
         finally:
-            if orig_username is not None:
-                os.environ[username_key] = orig_username
-            else:
-                os.environ.pop(username_key, None)
+            os.environ.clear()
+            os.environ.update(environ_copy)
 
     @classmethod
     def tearDownClass(cls):
@@ -443,7 +451,9 @@ class GitFSTestBase(object):
             try:
                 salt.utils.files.rm_rf(path)
             except OSError as exc:
-                if exc.errno != errno.EEXIST:
+                if exc.errno == errno.EACCES:
+                    log.error("Access error removeing file %s", path)
+                elif exc.errno != errno.EEXIST:
                     raise
 
     def setUp(self):
@@ -464,8 +474,14 @@ class GitFSTestBase(object):
             try:
                 salt.utils.files.rm_rf(os.path.join(self.tmp_cachedir, subdir))
             except OSError as exc:
+                if exc.errno == errno.EACCES:
+                    log.warning("Access error removeing file %s", os.path.join(self.tmp_cachedir, subdir))
+                    continue
                 if exc.errno != errno.ENOENT:
                     raise
+        if salt.ext.six.PY3 and salt.utils.platform.is_windows():
+            self.setUpClass()
+            self.setup_loader_modules()
 
 
 @skipIf(not HAS_GITPYTHON, 'GitPython >= {0} required'.format(GITPYTHON_MINVER))
@@ -486,6 +502,7 @@ class GitPythonTest(GitFSTestBase, GitFSTestFuncs, TestCase, LoaderModuleMockMix
 
 @skipIf(not HAS_GITPYTHON, 'GitPython >= {0} required for temp repo setup'.format(GITPYTHON_MINVER))
 @skipIf(not HAS_PYGIT2, 'pygit2 >= {0} and libgit2 >= {1} required'.format(PYGIT2_MINVER, LIBGIT2_MINVER))
+@skipIf(salt.utils.platform.is_windows(), 'Skip Pygit2 on windows, due to pygit2 access error on windows')
 @skipIf(NO_MOCK, NO_MOCK_REASON)
 class Pygit2Test(GitFSTestBase, GitFSTestFuncs, TestCase, LoaderModuleMockMixin):
 
